import { NS } from "@ns"

type FilePackEntry = {
    path: string;
    content: string;
};

type FilePack = FilePackEntry[];

const createScriptContent: (f: string) => string = (funcName: string) => `export async function main(ns) {
    if(ns.args.length < 3){
        ns.print("[ERROR] args required");
        ns.print("run cmd [sleepDuration] [hostname] [effectStock]")
        ns.exit();
        return;
    }
    const sleepDuration = parseInt(ns.args[0]);
    const hostname = ns.args[1].toString();
    const stock = ns.args[2] == "true";
    await ns.sleep(sleepDuration);
    await ns.${funcName}(hostname,{
        stock,
    });
}`

async function unpackFileEntry(ns: NS, filePack: FilePackEntry): Promise<void> {
    await ns.write(filePack.path, filePack.content, "w");
}

export async function unpackFilePack(ns: NS, pack: FilePack): Promise<void> {
    for (const entry of pack) {
        await unpackFileEntry(ns, entry);
    }
}

export const batchFilePack: FilePack = [
    {
        path: "/tmp/hack.js",
        content: createScriptContent("hack")
    },
    {
        path: "/tmp/weaken.js",
        content: createScriptContent("weaken")
    },
    {
        path: "/tmp/grow.js",
        content: createScriptContent("grow")
    }
]

export async function scpBatchFilePack(ns: NS,hostname :string): Promise<void> {
    const list = batchFilePack.map(entry=>entry.path);
    await installBatchFilePack(ns);
    await ns.scp(list,hostname);
}

export async function installBatchFilePack(ns: NS): Promise<void> {
    await unpackFilePack(ns, batchFilePack);
}

export function isBatchFilePackInstalled(ns: NS, hostname?: string): boolean {
    return batchFilePack.every(entry => ns.fileExists(entry.path, hostname))
}

export interface ExecOption{
    hostname: string; 
    /**
     * @default 1
     */
    numThread?: number;
    /**
     * @default 0
     */
    sleepDuration?: number;
    /**
     * target hostname to operate "hack", "weaken" or "grow"
     */
    target: number; 
    stock?: boolean;
}

function execBatchfile(ns: NS, path: string, option:ExecOption): number {
    return ns.exec(path, option.hostname, option.numThread,
        option.sleepDuration ?? 0, option.target, option.stock ?? false);
}

export function execHack(ns: NS,option: ExecOption): number{
    return execBatchfile(ns,"/tmp/hack.js",option);
}
export function execGrow(ns: NS,option: ExecOption): number{
    return execBatchfile(ns,"/tmp/grow.js",option);
}
export function execWeaken(ns: NS,option: ExecOption): number{
    return execBatchfile(ns,"/tmp/weaken.js",option);
}