import { selectRootedServerList, ServerInfo } from "lib/servers";
import {
    calculateWeakenTime,
    calculatePercentMoneyHacked,
    calculateServerGrowth
} from "lib/formula";
import { NS } from '@ns'
import { sprintf } from "./lib/printf";


// eslint-disable-next-line require-await
export async function main(ns: NS): Promise<void> {
    const list = selectRootedServerList(ns);
    const player = ns.getPlayer();
    const m = list.map(x => {
        const optimalState: ServerInfo = {...x,
            hackDifficulty: x.minDifficulty
        } as ServerInfo;
        ns.print(optimalState.minDifficulty," ", optimalState.hackDifficulty ,"");
        const weakenTime = calculateWeakenTime(optimalState, player);
        const earnMoney = calculatePercentMoneyHacked(optimalState, player);
        //const growPercent = calculateServerGrowth()
        return {
            hostname: x.hostname,
            info: x,
            weakenTime,
            earnMoney,
            ce: earnMoney* x.moneyMax / weakenTime,
        }
    });
    m.sort((a,b)=>(b.ce-a.ce));
    
    m.filter(x=>x.ce > 0).forEach(x=>{
        const msg = sprintf("%20s %8s %6.1fs %10.2f",x.hostname,
        ns.nFormat(x.earnMoney * x.info.moneyMax,"$0.00a"),x.weakenTime,
        x.ce*100);
        ns.tprint(msg);
    })
}